#!/usr/bin/python

# Copyright Abel Deuring 2013
#
# tftools is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

import argparse
from collections import namedtuple
from functools import partial
import sys
from brick_factory import BrickClassRegistry, BrickFactoryError
from time import sleep
import tftools
import tinkerforge
from tinkerforge.ip_connection import IPConnection


def get_params(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-H', '--host', dest='host', default='localhost')
    parser.add_argument(
        '-p', '--port', type=int, dest='port', default=4223)
    parser.add_argument(
        '-w', '--wait', type=float, dest='wait', default=0.5)
    return parser.parse_args(argv[1:])


DeviceInfo = namedtuple(
    'DeviceInfo',
    ('uid', 'connected_uid', 'position', 'hw_version', 'fw_version',
            'dev_id', 'enum_type'))


def cb_enum(uid_device, uid, connected_uid, position, hw_version, fw_version,
            dev_id, enum_type):
    uid_device[uid] = DeviceInfo(
        uid, connected_uid, position, hw_version, fw_version, dev_id, enum_type)


def devices_by_class(uid_device, registry):
    result = {}
    for uid, info in uid_device.items():
        try:
            classes = registry.cls_for_type(info.dev_id)
            class_names = [cls.__name__ for cls in classes]
        except BrickFactoryError:
            class_names = ['Unknown']
        for class_name in class_names:
            result.setdefault(class_name, []).append(info)
    return result


def main(argv=sys.argv):
    registry = BrickClassRegistry([tinkerforge, tftools])
    args = get_params(argv)
    conn = IPConnection()
    conn.connect(args.host, args.port)
    uid_device = {}
    conn.register_callback(
        conn.CALLBACK_ENUMERATE, partial(cb_enum, uid_device))
    conn.enumerate()
    sleep(args.wait)
    by_class = devices_by_class(uid_device, registry)
    for class_name, info in sorted(by_class.items()):
        for device in info:
            print "%s %s" % (class_name, device.uid)
    return 0


if __name__ == '__main__':
    sys.exit(main())
